/*
 * Copyright 2016 University of Chicago
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "globus_gridftp_server_s3_base.h"

/**
 * @brief Base64 encode an mdsum string's value
 * @details
 * This function takes a string representation of the hex value of an md5
 * checksum and converts it to the binary value and then base 64 encodes it
 * so that it can be sent as the value of a Content-MD5 header.
 * @param[in] md5sum
 *      A 32 byte string containing the hex representation of the md5 checksum.
 * @param[out] out
 *      An array of at least 25 bytes to hold the encoded checksum plus null
 *      terminator.
 */
globus_result_t
globus_i_s3_md5sum_encode(
    const char                          md5sum[static 32],
    char                                out[static 25])
{
    char                                encoder[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";
    unsigned char                       md5_binary[16];
    unsigned char                       c = 0;
    char                                bytepair[3] = {0};
    uint8_t                             byteval;
    size_t                              i, j;
    globus_result_t                     result = GLOBUS_SUCCESS;

    GlobusS3Enter();

    for (i = 0, j=0; i < 32; i++)
    {
        bytepair[0] = md5sum[i++];
        bytepair[1] = md5sum[i];

        if (!isxdigit(bytepair[0]) || (!isxdigit(bytepair[1])))
        {
            char fmt[] ="Invalid character in md5sum %32s";
            char msg[sizeof(fmt) + 33];
            sprintf(msg, fmt, md5sum);
            result = GlobusGFSErrorGeneric(msg);
            goto fail;
        }

        sscanf(bytepair, "%"SCNx8, &byteval);
            assert(isxdigit(md5sum[i]));

        md5_binary[j++] = byteval;
    }

    for (i=0, j=0; i < sizeof(md5_binary); i++)
    {
        switch (i % 3)
        {
            case 0:
                out[j++] = encoder[md5_binary[i]>>2];
                c = (md5_binary[i]&3)<<4;
                break;
            case 1:
                out[j++] = encoder[c|md5_binary[i]>>4];
                c = (md5_binary[i]&15)<<2;
                break;
            case 2:
                out[j++] = encoder[c|md5_binary[i]>>6];
                out[j++] = encoder[md5_binary[i]&63];
                c = 0;
                break;
        }
    }
    if (i%3)
    {
        out[j++] = encoder[c];
    }
    switch (i%3)
    {
        case 1:
            out[j++] = '=';
        case 2:
            out[j++] = '=';
    }

    out[j] = '\0';

    GlobusS3Trace("Encoded md5sum=%32s to %s\n", md5sum, out);
    GlobusS3ExitResult(result);

fail:
    return result;
}
/* globus_i_s3_md5sum_encode() */
// vim: ft=c:
